using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using System.Collections;


using Microsoft.WindowsMobile.Samples.Location;
using GpsSample;

using SharpMap.Geometries;
using SharpMap.Layers;
using SharpMap.Data.Providers;
using SharpMap.CoordinateSystems;
using SharpMap.CoordinateSystems.Transformations;

//Pour l'envoi vers le serveur
using positionLib;

namespace GpsTest
{
    
    /// <summary>
    /// IHM du Gps
    /// </summary>
    public partial class Form1 : System.Windows.Forms.Form, IDisposable
    {
        Gps gps = new Gps();
        GpsSample.Gpx _gpx = null;

        private GpsPosition lastpos = null;

        SharpMap.Map myMap;
        SharpMap.Layers.VectorLayer vLayer;

        List<Geometry> _parcours;
        List<GpsPosition> optimPoints = new List<GpsPosition>();

        List<SharpMap.Geometries.Point> _points = new List<SharpMap.Geometries.Point>();

        private bool visu = true;
        private double seuil = 0.0;

        //Pour prendre les points 2  2
        int nextpair = 0;

        private bool saveAsGpx = false;
        private bool envoye = false;

        #region Constructors
        public Form1()
        {
            //
            // Requis pour le Windows Form Designer
            //
            InitializeComponent();

            myMap = new SharpMap.Map(new Size(this.Width, this.Height));
            myMap.BackColor = Color.LightBlue;
            
            SharpMap.Layers.VectorLayer myLayer = new SharpMap.Layers.VectorLayer("Boundaries");
            myLayer.DataSource = new SharpMap.Data.Providers.ShapeFile("Program Files\\GpsSample\\France\\france.shp");
            myLayer.Style.Line.Width = 2;
            myLayer.Style.Line.Color = Color.Black;
            myLayer.Style.Fill = new SolidBrush(Color.LightGreen);
            myLayer.Style.EnableOutline = true;
            //myLayer.MaxVisible = 40000;
            myLayer.MaxVisible = 40;
            myMap.Layers.Add(myLayer);

            //Nom des rgions
            SharpMap.Layers.LabelLayer labelLayer = new SharpMap.Layers.LabelLayer("Country Names");
            labelLayer.DataSource = myLayer.DataSource;
            labelLayer.LabelColumn = "ADMIN_NAME";
            labelLayer.Style.CollisionDetection = true;
            labelLayer.Style.Font = new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular);
            labelLayer.Style.CollisionBuffer = new Size(10, 10);
            myMap.Layers.Add(labelLayer);

            vLayer = new SharpMap.Layers.VectorLayer("GeometryLayer");
            _parcours = new List<Geometry>();

            vLayer.DataSource = new SharpMap.Data.Providers.GeometryProvider(_parcours);
            vLayer.Style.Enabled = true;
            vLayer.Style.Line = new Pen(Color.Blue, 2);
            Bitmap bmp = new Bitmap(3, 3);
            bmp.SetPixel(0, 0, Color.Red);
            bmp.SetPixel(1, 1, Color.Red);
            bmp.SetPixel(2, 2, Color.Red);
            vLayer.Style.Symbol = bmp;

            myMap.Layers.Add(vLayer);

            myMap.ZoomToExtents();
            try
            {
                this.imgMap.Map = myMap;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Erreur : mmoire insuffisante " + ex.GetBaseException(), "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
            }
        }
        #endregion

        private void resetPath()
        {
            _points.Clear();
            optimPoints.Clear();
            _parcours.Clear();
            this.evtLog.Clear();
            try
            {
                this.imgMap.Refresh();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Erreur : mmoire insuffisante " + ex.GetBaseException(), "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
            }
            nextpair = 0;
            envoye = false;
        }

     

        /// <summary>
        /// The main entry point for the application.
        /// </summary>

        static void Main()
        {
            Application.Run(new Form1());
        }

        #region event handlers
        private void exitMenuItem_Click(object sender, EventArgs e)
        {
            if (gps.Opened)
            {
                gps.Close();
            }

            Close();
        }

        private void Form1_Load(object sender, System.EventArgs e)
        {

            updateDataHandler = new EventHandler(UpdateData);
            gps.DeviceStateChanged += new DeviceStateChangedEventHandler(gps_DeviceStateChanged);
            gps.LocationChanged += new LocationChangedEventHandler(gps_LocationChanged);
        }

        protected void gps_LocationChanged(object sender, LocationChangedEventArgs args)
        {
            position = args.Position;

            // call the UpdateData method via the updateDataHandler so that we
            // update the UI on the UI thread
            Invoke(updateDataHandler);
        }

        void gps_DeviceStateChanged(object sender, DeviceStateChangedEventArgs args)
        {
            device = args.DeviceState;
            // call the UpdateData method via the updateDataHandler so that we
            // update the UI on the UI thread
            Invoke(updateDataHandler);
        }
  
        void UpdateData(object sender, System.EventArgs args)
        {
            if (gps.Opened)
            {
               
                if (device.FriendlyName!=null)
                    this.txtDevName.Text = device.FriendlyName;

                if (device.ServiceState.ToString().Equals("On") & device.DeviceState.ToString().Equals("On"))
                {
                    this.radioOnOff.Checked = true;
                    this.radioOnOff.Text = device.ServiceState.ToString();
                    this.radioOnOff.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(0)), ((System.Byte)(192)), ((System.Byte)(0)));
                }
                else
                {
                    this.radioOnOff.Checked = false;
                    this.radioOnOff.Text = device.ServiceState.ToString();
                    this.radioOnOff.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(192)), ((System.Byte)(0)), ((System.Byte)(0)));
                }


                if (position != null)
                {
                    if (position.SatelliteCount > 0 && position.GetSatellitesInSolution() != null)
                    {
                        DrawGSV();
                    }

                    

                        if (saveAsGpx)
                        {
                            if (_gpx == null)
                            {
                                _gpx = new GpsSample.Gpx(position, this.GPXfile);
                            }
                            else
                            {
                                _gpx.SetPos(position);
                                _gpx.GetGpx();
                            }

                            this.GPXfile.Flush();
                            Update_Size(savGPX, "GPX");
                        } 


                        if ((position.LatitudeValid) && (position.Latitude!=0))
                        {
                            
                            GpsPosition currentpos = position;

                            this.txtLatitude.Text = currentpos.Latitude.ToString();

                            if ((position.LongitudeValid) && (position.Longitude!=0))
                            {
                                this.txtLongitude.Text = currentpos.Longitude.ToString();
                                
                                //GPRMC
                                position.WriteGPRMC(GPRMCfile);
                                GPRMCfile.Flush();
                                Update_Size(savGPRMC, "GPRMC");

                                // On ajoute le point au parcours pour la Route
                                _parcours.Add(new SharpMap.Geometries.Point(position.Longitude, position.Latitude));
                                _points.Add(new SharpMap.Geometries.Point(position.Longitude, position.Latitude));

                                if (lastpos != null)
                                {
                                    //if ((nextpair % 2) == 0)
                                    //{
                                        // On est sur la prochaine paire
                                        Optim opt = new Optim(lastpos, currentpos, seuil);
                                        opt.WriteGPRMC(Optimfile, this.visu);
                                        //opt.Optimize(Optimfile, this.visu);
                                        if (opt.Keep)
                                        {
                                            optimPoints.Add(opt.Pos1);
                                            //optimPoints.Add(opt.Pos2);
                                        }
                                        Optimfile.Flush();
                                        Update_Size(savOptim, "OPTIM");
                                    //}
                                    List<SharpMap.Geometries.Point> Line = new List<SharpMap.Geometries.Point>();
                                    Line.Add(new SharpMap.Geometries.Point(lastpos.Longitude, lastpos.Latitude));
                                    Line.Add(new SharpMap.Geometries.Point(position.Longitude, position.Latitude));
                                    
                                    _parcours.Add(new SharpMap.Geometries.LineString(Line));
                                    
                                    if (position.HeadingValid)
                                    {
                                        textBoxHeading.Text = position.Heading.ToString();
                                        //Pour la boussole
                                        pictureBoxPoint.Left = (int)(116 + 100 * Math.Sin(position.Heading * Math.PI / 180));
                                        pictureBoxPoint.Top = (int)(116 - 100 * Math.Cos(position.Heading * Math.PI / 180));
                                    }
                                }
                                lastpos = currentpos;
                                nextpair++;
                            }
                        }
                        

                        if (position.TimeValid)
                        {
                            evtLog.Add(position.Time, position);
                        }
                }
            }
        }

        private void Update_Size(SaveFileDialog fd, String type) {
            
            long sizeOctets = new FileInfo(fd.FileName).Length;
            Double sizeKo = Math.Round((Double)sizeOctets / 1000, 2);
            
            if (type.Equals("GPRMC")) {
                labTailleGPRMC.Text = string.Format("Taille : {0} Ko", sizeKo);
                }
            else {
                if (type.Equals("GPX"))
                {
                    this.labTailleGPX.Text = string.Format("Taille : {0} Ko", sizeKo);
                } 
                else
                    if (type.Equals("OPTIM"))
                    {
                        this.labTailleOptim.Text = string.Format("Taille : {0} Ko", sizeKo);
                    }
                }
        }



        private void Form1_Closed(object sender, System.EventArgs e)
        {
            stopGpsMenuItem_Click(sender, e);
            /*if (gps.Opened)
            {
                gps.Close();
            }

            if (_gpx!=null)
                _gpx.CloseFile();

            if (GPRMCfile != null)
                GPRMCfile.Close();*/
        }

        private void stopGpsMenuItem_Click(object sender, EventArgs e)
        {
            if (gps.Opened)
            {
                gps.Close();
            }

            //Pour crire le dernier point
            //if ((position!=null) && !envoye)
                //position.WriteGPRMC(Optimfile);
            
            this.txtOptim.Enabled = false;
            this.txtGPRMC.Enabled = false;
            this.txtGPX.Enabled = false;
            this.chkVisu.Enabled = true;
            this.chkEchanti.Enabled = true;
            this.txtSeuil.Enabled = true;

            this.radioOnOff.Checked = false;
            this.radioOnOff.Text = "Off";
            this.radioOnOff.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(192)), ((System.Byte)(0)), ((System.Byte)(0)));

            startGpsMenuItem.Enabled = true;
            stopGpsMenuItem.Enabled = false;

            if (saveAsGpx)
            {
                _gpx.CloseFile();
                _gpx = null;
            }
            if (GPRMCfile!=null)
                GPRMCfile.Close();
            if (Optimfile != null)
                Optimfile.Close();
            this.txtGPRMC.Text = "";
            this.txtGPX.Text = "";
            this.txtOptim.Text = "";

            if ((!envoye) && (optimPoints.Count>0))
            {
                //Pour envoyer les donnes au serveur les donnes au serveur
                positionLib.PositionFunctions posit;
                try
                {

                    posit = new positionLib.PositionFunctions();
                    /*** to use Active Sync, take this address : 192.168.55.100 ***/
                    posit.codePosition(optimPoints, "192.168.55.100");
                    MessageBox.Show("encoding and sending succeeded");

                }
                catch (Exception e1)
                {
                    MessageBox.Show(e1.Message);
                    return;
                }
                envoye = true;
            }
        }

        private void startGpsMenuItem_Click(object sender, EventArgs e)
        {
            this.resetPath();
            

            bool gprmc_ok = true;
            bool optim_ok = true;

            if (this.txtGPX.Text.Equals("") || this.txtGPX.Text.Equals(String.Empty))
            {
                DialogResult res = MessageBox.Show("Voulez-vous sauver le contenu brut au format GPX?", "Sauvegarde fichier GPX", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
                if (res == DialogResult.Yes)
                    saveAsGpx = true;

                if (saveAsGpx)
                {
                    this.btnLogGPX_Click(sender, e);
                    /* if (savGPX.ShowDialog() == DialogResult.OK)
                     {
                         this.GPXfile = new StreamWriter(savGPX.FileName);
                         this.txtGPX.Text = savGPX.FileName;
                     }*/
                }
            }

            if (this.txtGPRMC.Text.Equals("") || this.txtGPRMC.Text.Equals(String.Empty))
                gprmc_ok = this.logGPRMCShow();

            if (this.txtOptim.Text.Equals("") || this.txtOptim.Text.Equals(String.Empty))
                optim_ok = this.logOptimShow();

            /*if (savGPRMC.ShowDialog() == DialogResult.OK)
            {
                 GPRMCfile = new StreamWriter(savGPRMC.FileName);
                 this.txtGPRMC.Text = savGPRMC.FileName;
            }

            if (savOptim.ShowDialog() == DialogResult.OK)
            {
                Optimfile = new StreamWriter(savOptim.FileName);
                this.txtOptim.Text = this.savOptim.FileName;
            }
            else
            {
                return;
            }*/

            if (!gps.Opened && (gprmc_ok) && (optim_ok))
            {
                gps.Open();
                startGpsMenuItem.Enabled = false;
                stopGpsMenuItem.Enabled = true;
                this.txtOptim.Enabled = false;
                this.txtGPRMC.Enabled = false;
                this.txtGPX.Enabled = false;
                this.chkVisu.Enabled = false;
                this.txtSeuil.Enabled = false;
                this.chkEchanti.Enabled = false;
            }
        }

        private void DrawGSV()
        {
            System.Drawing.Color[] Colors = { Color.Blue , Color.Red , Color.Green, Color.Yellow, Color.Cyan, Color.Orange,
											  Color.Gold , Color.Violet, Color.YellowGreen, Color.Brown, Color.GreenYellow,
											  Color.Blue , Color.Red , Color.Green, Color.Yellow, Color.Aqua, Color.Orange};
            //Generate signal level readout
            int SatCount = position.SatelliteCount;
            Bitmap imgSignals = new Bitmap(picGSVSignals.Width, picGSVSignals.Height);
            Graphics g = Graphics.FromImage(imgSignals);
            g.Clear(Color.White);
            Pen penBlack = new Pen(Color.Black, 1);
            Pen penGray = new Pen(Color.LightGray, 1);
            int iMargin = 4; //Distance to edge of image
            int iPadding = 4; //Distance between signal bars
            g.DrawRectangle(penBlack, 0, 0, imgSignals.Width - 1, imgSignals.Height - 1);

            StringFormat sFormat = new StringFormat();
            int barWidth = 1;
            if (SatCount > 0)
                barWidth = (imgSignals.Width - 2 * iMargin - iPadding * (SatCount - 1)) / SatCount;

            //Draw horisontal lines
            for (int i = imgSignals.Height - 15; i > iMargin; i -= (imgSignals.Height - 15 - iMargin) / 5)
                g.DrawLine(penGray, 1, i, imgSignals.Width - 2, i);
            sFormat.Alignment = StringAlignment.Center;
            //Draw satellites
            Satellite[] sat = position.GetSatellitesInView();
            for (int i = 0; i < SatCount; i++)
            {
                int startx = i * (barWidth + iPadding) + iMargin;
                int starty = imgSignals.Height - 15;
                int height = (imgSignals.Height - 15 - iMargin) / 50 * sat[i].SignalStrength;
                g.FillRectangle(new System.Drawing.SolidBrush(Colors[i]), startx, starty, barWidth, -(sat[i].SignalStrength));
                                
                if ((position.GetListOfSatellitesInSolution()).Contains(sat[i].Id))
                {
                    g.DrawRectangle(penBlack, startx, starty, barWidth, -height);
                }

                
                //if (((List <Satellite>)position.GetSatellitesInSolution()).Contains(sat[i].Id))
                //g.DrawRectangle(penBlack, startx, starty, barWidth, -height);

                sFormat.LineAlignment = StringAlignment.Near;
                g.DrawString(sat[i].Id.ToString(), new Font("Verdana", 9, FontStyle.Regular), new System.Drawing.SolidBrush(Color.Black), startx + barWidth / 2, imgSignals.Height - 15, sFormat);
                sFormat.LineAlignment = StringAlignment.Far;
                g.DrawString(sat[i].SignalStrength.ToString(), new Font("Verdana", 9, FontStyle.Regular), new System.Drawing.SolidBrush(Color.Black), startx + barWidth / 2, starty - height, sFormat);
            }
            picGSVSignals.Image = imgSignals;

            //Generate sky view
            Bitmap imgSkyview = new Bitmap(picGSVSkyview.Width, picGSVSkyview.Height);
            g = Graphics.FromImage(imgSkyview);
            g.Clear(Color.White);
            g.DrawEllipse(penGray, 0, 0, imgSkyview.Width - 1, imgSkyview.Height - 1);
            g.DrawEllipse(penGray, imgSkyview.Width / 4, imgSkyview.Height / 4, imgSkyview.Width / 2, imgSkyview.Height / 2);
            g.DrawLine(penGray, imgSkyview.Width / 2, 0, imgSkyview.Width / 2, imgSkyview.Height);
            g.DrawLine(penGray, 0, imgSkyview.Height / 2, imgSkyview.Width, imgSkyview.Height / 2);
            sFormat.LineAlignment = StringAlignment.Near;
            sFormat.Alignment = StringAlignment.Near;
            for (int i = 0; i < SatCount; i++)
            {
                double ang = 90.0 - sat[i].Azimuth;
                ang = ang / 180.0 * Math.PI;
                int x = imgSkyview.Width / 2 + (int)Math.Round((Math.Cos(ang) * ((90.0 - sat[i].Elevation) / 90.0) * (imgSkyview.Width / 2.0 - iMargin)));
                int y = imgSkyview.Height / 2 - (int)Math.Round((Math.Sin(ang) * ((90.0 - sat[i].Elevation) / 90.0) * (imgSkyview.Height / 2.0 - iMargin)));
                g.FillEllipse(new System.Drawing.SolidBrush(Colors[i]), x - 1, y - 1, 3, 3);
                g.DrawString(sat[i].Id.ToString(), new Font("Verdana", 9, FontStyle.Regular), new System.Drawing.SolidBrush(Color.Black), x, y, sFormat);
            }
            picGSVSkyview.Image = imgSkyview;
        }


        private void chkVisu_CheckStateChanged(object sender, EventArgs e)
        {
            if (!gps.Opened)
            {
                this.visu = chkVisu.Checked;
            }
            else
            {
                MessageBox.Show("Vous ne pouvez modifier cette valeur que lorsque le GPS est arrt!", "Attention", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
            }
        }

        private bool logGPRMCShow()
        {
            bool gprmc_ok = false;
            if (!gps.Opened)
            {
                savGPRMC.FileName = "GPRMC_" + DateTime.Now.Day + "" + DateTime.Now.Month + "" + (DateTime.Now.Year).ToString().Substring(2, 2) + "_" + DateTime.Now.Hour + "" + DateTime.Now.Minute + "" + DateTime.Now.Second;
                if (this.savGPRMC.ShowDialog() == DialogResult.OK)
                {
                    this.GPRMCfile = new StreamWriter(savGPRMC.FileName);
                    this.txtGPRMC.Text = savGPRMC.FileName;
                    gprmc_ok = true;
                }
                else
                {
                    gprmc_ok = false;
                }
            }
            else
            {
                MessageBox.Show("Vous ne pouvez modifier le fichier de sauvegarde que lorsque le GPS est arrt!", "Attention", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
            }
            return gprmc_ok;
        }

        private bool logOptimShow()
        {
            bool optim_ok = false;
            if (!gps.Opened)
            {
                savOptim.FileName = "OPTIM_" + DateTime.Now.Day + "" + DateTime.Now.Month + "" + (DateTime.Now.Year).ToString().Substring(2, 2) + "_" + DateTime.Now.Hour + "" + DateTime.Now.Minute + "" + DateTime.Now.Second;
                if (this.savOptim.ShowDialog() == DialogResult.OK)
                {
                    this.Optimfile = new StreamWriter(savOptim.FileName);
                    this.txtOptim.Text = savOptim.FileName;
                    optim_ok = true;
                }
                else
                {
                    optim_ok = false;
                }
            }
            else
            {
                MessageBox.Show("Vous ne pouvez modifier le fichier de sauvegarde que lorsque le GPS est arrt!", "Attention", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
            }
            return optim_ok;
        }

        private void btnLogGPRMC_Click(object sender, EventArgs e)
        {
            this.logGPRMCShow();
        }

        private void btnLogOptim_Click(object sender, EventArgs e)
        {
            this.logOptimShow();
        }

        private void btnLogGPX_Click(object sender, EventArgs e)
        {
            if (!gps.Opened)
            {
                savGPX.FileName = "GPX_" + DateTime.Now.Day + "" + DateTime.Now.Month + "" + (DateTime.Now.Year).ToString().Substring(2, 2) + "_" + DateTime.Now.Hour + "" + DateTime.Now.Minute + "" + DateTime.Now.Second;
                if (this.savGPX.ShowDialog() == DialogResult.OK)
                {
                    this.GPXfile = new StreamWriter(savGPX.FileName);
                    this.txtGPX.Text = savGPX.FileName;

                }
            }
            else
            {
                MessageBox.Show("Vous ne pouvez modifier le fichier de sauvegarde que lorsque le GPS est arrt!", "Attention", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
            }
        }

        #endregion

        #region LoadSaveMenu

        /*private SharpMap.Geometries.Point findPoint(List<SharpMap.Geometries.Point> liste, SharpMap.Geometries.Point search) 
        {
            //TODO chercher le point le plus proche!
           /* foreach(SharpMap.Geometries.Point p in liste)
            {
                if (p.X == search.X && p.Y == search.Y)
                    return p;
            }
            return null;*/
        //}

        private void menuItemSavePath_Click(object sender, EventArgs e)
        {
            if (savGPX.ShowDialog() == DialogResult.OK)
            {
                Gpx gpx = null;
                this.GPXfile = new StreamWriter(savGPX.FileName);
                gpx = new GpsSample.Gpx(this.GPXfile);
                if (!this.txtSeuil.Text.Equals(""))
                {
                    foreach (GpsPosition opt_point in optimPoints)
                    {
                        gpx.SetPos(opt_point);
                        gpx.GetGpx();
                    }
                }
                else
                {
                    foreach (SharpMap.Geometries.Point point in _points)
                    {
                        GpsPosition gpos = new GpsPosition(point.Y, point.X);
                        gpx.SetPos(gpos);
                        gpx.GetGpx();
                    }
                }
                gpx.CloseFile();
            }
        }

        private void menuItemLoadPath_Click(object sender, EventArgs e)
        {
            DialogResult res;
            //int suivpair = 0;
            envoye = false;

            if (gps.Opened)
            {
                res = MessageBox.Show("tes-vous sr de vouloir continuer (le GPS sera arrt et le parcours rinitialis)?", "Arrt communication GPS", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
                if (res == DialogResult.Yes)
                    gps.Close();
            }

            if (!gps.Opened)
            {

                openFileTracer.ShowDialog();
                _points.Clear();
                optimPoints.Clear();
                _parcours.Clear();
                try
                {
                    this.imgMap.Refresh();
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Erreur : mmoire insuffisante " + ex.GetBaseException(), "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
                }

                GpsPosition last_pos = null;
                GpsPosition curr_pos = null;

                if (openFileTracer.FileName.Length > 0)
                {
                    TextReader fluxInfos = null;
                    string ligne = null;
                    // Match m;
                    // Regex nmea = new Regex(@"^\$GP(?<trame>[A-Z]{3}),(?<horaire>[0-9\.]+),(?<status>[A|V]),(?<latitude>[0-9\.]+),(?<dirlat>[N|S]),(?<longitude>[0-9\.]+),(?<dirlong>[E|W]),(?<vitesse>[0-9\.]+),(?<trackmadegood>[0-9\.]+),(?<date>[0-9]{6}),(?<magnetique>[0-9\.]+),(?<eW>[E|W])\*(?<checksum>[0-9A-Z]+)$");
                    using (fluxInfos = new StreamReader(openFileTracer.FileName))
                    {
                        ligne = fluxInfos.ReadLine();
                        while (ligne != null)
                        {
                            string[] Words = ligne.Split(',');
                            if (Words[0].Equals("$GPRMC"))
                            {
                                string latitude = Words[3];
                                string dirlat = Words[4];
                                string longitude = Words[5];
                                string dirlong = Words[6];
                                //string longitude = m.Groups[nmea.GroupNumberFromName("longitude")].Value;
                                //string dirlong = m.Groups[nmea.GroupNumberFromName("dirlong")].Value;
                                if (!latitude.Equals("") && !longitude.Equals(""))
                                {
                                    _parcours.Add(new SharpMap.Geometries.Point(DegreesMinutesSeconds.ToDecimalDegrees(longitude + dirlong), DegreesMinutesSeconds.ToDecimalDegrees(latitude + dirlat)));
                                    _points.Add(new SharpMap.Geometries.Point(DegreesMinutesSeconds.ToDecimalDegrees(longitude + dirlong), DegreesMinutesSeconds.ToDecimalDegrees(latitude + dirlat)));
                                    curr_pos = new GpsPosition(DegreesMinutesSeconds.ToDecimalDegrees(latitude + dirlat), DegreesMinutesSeconds.ToDecimalDegrees(longitude + dirlong));
                                }
                                if (last_pos != null)
                                {
                                    List<SharpMap.Geometries.Point> Line = new List<SharpMap.Geometries.Point>();
                                    Line.Add(new SharpMap.Geometries.Point(last_pos.Longitude, last_pos.Latitude));
                                    Line.Add(new SharpMap.Geometries.Point(curr_pos.Longitude, curr_pos.Latitude));
                                    _parcours.Add(new SharpMap.Geometries.LineString(Line));
                                    
                                    //Pour prendre pair  pair
                                    //if ((suivpair % 2) == 0)
                                    //{
                                        Optim opt = new Optim(last_pos, curr_pos, seuil);
                                        opt.Optimize();
                                        if (opt.Keep)
                                        {
                                            optimPoints.Add(opt.Pos1);
                                            //optimPoints.Add(opt.Pos2);
                                        }
                                    //}
                                }
                                //suivpair++;
                                last_pos = curr_pos;
                            }                     
                            ligne = fluxInfos.ReadLine();
                        }
                    }
                    optimPoints.Add(curr_pos);
                }
            }
        }

        #endregion

        #region IndexMgmt

        private void tabMain_SelectedIndexChanged(object sender, EventArgs e)
        {

            if (this.tabMain.SelectedIndex == this.tabMain.TabPages.IndexOf(this.tabPageRoute))
            {
                this.exitMenuItem.MenuItems.Add(this.menuItemCenter);
                this.exitMenuItem.MenuItems.Add(this.menuItemPath);
                this.exitMenuItem.Text = "Menu";
            }
            else
            {
 
                    this.exitMenuItem.MenuItems.Remove(this.menuItemCenter);
                    this.exitMenuItem.MenuItems.Remove(this.menuItemPath);
                    this.exitMenuItem.Text = "Quitter";
                
            }
        }

        #endregion

        #region Panning

        private void tabPageRoute_Click(object sender, EventArgs e)
        {
            this.lblLatLon.Visible = false;
            if (this.zoomIn.CheckState == CheckState.Checked) //Zoom in 
            {
                try
                {
                    this.imgMap.ActiveTool = SharpMap.Forms.MapImage.Tools.ZoomIn;
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Exception : "+ex, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
                }
                //Set center of the map to where the client clicked
                //myMap.Center = SharpMap.Utilities.Transform.MapToWorld(new System.Drawing.Point(Form.MousePosition.X, Form.MousePosition.Y), myMap);
                //Set zoom value if any of the zoom tools were selected
                //myMap.Zoom = myMap.Zoom * 0.5;
            }
            else if (this.zoomOut.CheckState == CheckState.Checked) //Zoom out
            {
                try
                {
                    this.imgMap.ActiveTool = SharpMap.Forms.MapImage.Tools.ZoomOut;
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Erreur : mmoire insuffisante "+ex.GetBaseException(), "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
                }
                //Set center of the map to where the client clicked
                //myMap.Center = SharpMap.Utilities.Transform.MapToWorld(new System.Drawing.Point(Form.MousePosition.X, Form.MousePosition.Y), myMap);
                //Set zoom value if any of the zoom tools were selected
                //myMap.Zoom = myMap.Zoom * 2;
            }
            else if (this.pan.CheckState == CheckState.Checked)
            {
                try
                {
                    this.imgMap.ActiveTool = SharpMap.Forms.MapImage.Tools.Pan;
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Erreur : mmoire insuffisante "+ex.GetBaseException(), "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
                }
                 //SharpMap.Geometries.Point p = myMap.ImageToWorld(new System.Drawing.Point(Form.MousePosition.X, Form.MousePosition.Y));
                 //myMap.Center.X = p.X;
                 //myMap.Center.Y = p.Y;
                 //this.imgMap.Refresh();
            }
            else
            {
                SharpMap.Geometries.Point search = myMap.ImageToWorld(new System.Drawing.Point(Form.MousePosition.X, Form.MousePosition.Y));
               // SharpMap.Geometries.Point found = findPoint(_points, search);
               // if (found!=null)
               //     this.lblLatLon.Text = "Longitude : " + Math.Round(found.X, 8) + " Latitude : " + Math.Round(found.Y, 8);
               // else
                this.lblLatLon.Text = "Longitude : " + Math.Round(search.X, 4) + " Latitude : " + Math.Round(search.Y, 4);
                this.lblLatLon.Visible = true;
            }
            try
            {
                this.imgMap.Refresh();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Erreur : mmoire insuffisante " + ex.GetBaseException(), "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
            }
        }

        private void menuItemCenter_Click(object sender, EventArgs e)
        {
            if (_points != null)
            {

                //Calculate bounding rectangle
                double maxLat = -90, minLat = 90, maxLon = -180, minLon = 180;

                //Calculate bounding box for array of locations
                for (int i = 0; i < _points.Count; i++)
                {
                    //get latitudes
                    if (_points[i].Y > maxLat)
                        maxLat = _points[i].Y;

                    if (_points[i].Y < minLat)
                        minLat = _points[i].Y;

                    //get longitudes
                    if (_points[i].X > maxLon)
                        maxLon = _points[i].X;

                    if (_points[i].X < minLon)
                        minLon = _points[i].X;
                }

                //calculate center coordinate of bounding box
               // double centerLat = (maxLat + minLat) / 2;
               // double centerLong = (maxLon + minLon) / 2;

                BoundingBox bbox = new BoundingBox(minLon, minLat, maxLon, maxLat);
                myMap.ZoomToBox(bbox);
               // myMap.Center.X = centerLong;
               // myMap.Center.Y = centerLat;

                try
                {
                    this.imgMap.Refresh();
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Erreur : mmoire insuffisante " + ex.GetBaseException(), "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
                }
            }
        }

        private void resetMap_Click(object sender, EventArgs e)
        {
            myMap.ZoomToExtents();
            try
            {
                this.imgMap.Refresh();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Erreur : mmoire insuffisante " + ex.GetBaseException(), "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
            }
        }

        

        private void zoomIn_CheckStateChanged(object sender, EventArgs e)
        {
            zoomOut.Checked = false;
            pan.Checked = false;
        }

        private void zoomOut_CheckStateChanged(object sender, EventArgs e)
        {
            zoomIn.Checked = false;
            pan.Checked = false;
        }

        private void pan_CheckStateChanged(object sender, EventArgs e)
        {
            zoomIn.Checked = false;
            zoomOut.Checked = false;
        }

        #endregion

        private void txtSeuil_TextChanged(object sender, EventArgs e)
        {
            if (!gps.Opened)
            {
                bool percent = false;
                //MessageBox.Show("Info : de 0.0  0.99 ou la valeur contient '%' alors en pourcentage sinon en degres", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                string stVal = this.txtSeuil.Text;
                if (stVal.Contains("%"))
                {
                    stVal = stVal.Replace("%", "");
                    percent = true;
                }

                try
                {
                    this.seuil = Convert.ToDouble(stVal);
                    if (percent)
                    {
                        this.seuil = (this.seuil) / 100;
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("La valeur rentre est incorrecte !", "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                }
            }
            else
            {
                MessageBox.Show("Vous ne pouvez modifier cette valeur que lorsque le GPS est arrt!", "Attention", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                this.txtSeuil.Text = this.seuil.ToString();
            }
        }
    }
}
