﻿using System;

using System.Collections.Generic;
using System.Text;
using System.IO;

using Microsoft.WindowsMobile.Samples.Location;


namespace GpsSample
{
    class Optim
    {
        internal class Degrees {
            private Double value;
            public Degrees(Double value)
            {
                if (value > 360.0 || value < 0.0)
                {
                    throw new ArgumentException();
                }
                this.value = value;
            }

            /// <summary>
            /// Converts the instance to a string"
            /// </summary>
            /// <returns>string representation of degrees</returns>
            public override string ToString()
            {
                return this.value.ToString() + "° ";
            }

            public static explicit operator Degrees(Double c)
            {
                return new Degrees(c);
            }

            public static explicit operator Double(Degrees c)
            {
                return c.value;
            }

        }

        private bool keep = false;

        public bool Keep
        {
            get { return keep; }
        }

        private Double seuil = 0.0;

        public Double Seuil
        {
            get { return seuil; }
            set
            {
                    this.seuil = value;
            }
        }

        private static Double AncBear;
        private Double NewBear;

        private GpsPosition pos1;

        public GpsPosition Pos1
        {
            get { return pos1; }
            set { pos1 = value; }
        }

        private GpsPosition pos2;

        public GpsPosition Pos2
        {
            get { return pos2; }
            set { pos2 = value; }
        }

        public GpsPosition GetPosition1()
        {
            return this.pos1;
        }

        public GpsPosition GetPosition2()
        {
            return this.pos2;
        }

        public void SetPosition1(GpsPosition pos)
        {
            this.pos1 = pos;
        }

        public void SetPosition1(Double lat, Double lon)
        {
            this.pos1 = new GpsPosition(lat, lon);
        }

        public void SetPosition2(GpsPosition pos)
        {
            this.pos2 = pos;
        }

        public void SetPosition2(Double lat, Double lon)
        {
            this.pos2 = new GpsPosition(lat, lon);
        }

        public Optim(Double lat1, Double lon1, Double lat2, Double lon2)
        {
            SetPosition1(lat1, lon1);
            SetPosition2(lat2, lon2);
        }


        public Optim(GpsPosition pos1, GpsPosition pos2)
        {
            this.pos1 = pos1;
            this.pos2 = pos2;
        }

        public Optim(GpsPosition pos1, GpsPosition pos2, double seuil)
        {
            this.pos1 = pos1;
            this.pos2 = pos2;
            this.Seuil = seuil;
        }

        public Optim(Optim o)
        {
            this.pos1 = o.pos1;
            this.pos2 = o.pos2;
        }

        public Degrees Bearing() {
            
            Degrees deg = (Degrees) 0.0;
            Double dLon = GpsPosition.DegreeToRadian(this.pos2.Longitude - this.pos1.Longitude);
            Double y = Math.Sin(dLon) * Math.Cos(this.pos2.LatitudeRad);
            Double x = Math.Cos(this.pos1.LatitudeRad) * Math.Sin(this.pos2.LatitudeRad) - Math.Sin(this.pos1.LatitudeRad) * Math.Cos(this.pos2.LatitudeRad) * Math.Cos(dLon);

            if (x == 0 && y == 0)
            {
                deg =  (Degrees) 0.0;
            }
            else
            {
                deg = (Degrees) (((GpsPosition.RadianToDegree(Math.Atan2(y, x))) + 360) % 360);
            }
            return deg;
        }

        public void Optimize(params Object[] parametres)
        {
            if (pos1.Latitude != pos2.Latitude && pos1.Longitude != pos2.Longitude)
            {
                this.NewBear = (Double)this.Bearing();

                if (this.seuil < 0.9999999)
                {
                    //Seuil en pourcentage
                    if (NewBear <= AncBear * (1 - this.seuil) || NewBear >= AncBear * (1 + this.seuil))
                    {
                        if (parametres.Length == 0)
                            this.keep = true;
                        else
                        {
                            //On écrit dans le fichier
                            if ((bool) parametres[1])
                            {
                                pos1.WriteGPRMC((TextWriter) parametres[0]);
                                //pos2.WriteGPRMC((TextWriter)parametres[0]);
                            }
                            else
                            {
                                ((TextWriter)parametres[0]).WriteLine(pos1.Latitude + "," + pos1.Longitude);
                                //((TextWriter)parametres[0]).WriteLine(pos2.Latitude + "," + pos2.Longitude);
                            }
                        }
                        // On prend la nouvelle valeur comme référence
                        AncBear = NewBear;
                    }
                    else
                    {
                        if (parametres.Length == 0)
                            this.keep = false;
                    }
                }
                else
                {
                    //Seuil en degrees
                    if (NewBear <= (AncBear - this.seuil) || NewBear >= (AncBear + this.seuil))
                    {
                        if (parametres.Length == 0)
                        {
                            this.keep = true;
                        }
                        else
                        {
                            //On écrit dans le fichier
                            if ((bool)parametres[1])
                            {
                                pos1.WriteGPRMC((TextWriter)parametres[0]);
                                //pos2.WriteGPRMC((TextWriter)parametres[0]);
                            }
                            else
                            {
                                ((TextWriter)parametres[0]).WriteLine(pos1.Latitude + "," + pos1.Longitude);
                                //((TextWriter)parametres[0]).WriteLine(pos2.Latitude + "," + pos2.Longitude);
                            }
                        }
                        // On prend la nouvelle valeur comme référence
                        AncBear = NewBear;
                    }
                    else
                    {
                        if (parametres.Length == 0)
                            this.keep = false;
                    }
                }
            }
        } 

        public void WriteGPRMC(TextWriter output, bool visu)
        {
            if (pos1.Latitude != pos2.Latitude && pos1.Longitude != pos2.Longitude)
            {
                this.NewBear = (Double) this.Bearing();

                if (this.seuil < 0.9999999)
                {
                    //Seuil en pourcentage
                    if (NewBear <= AncBear * (1 - this.seuil) || NewBear >= AncBear * (1 + this.seuil))
                    {
                        this.keep = true;
                        //On écrit dans le fichier
                        if (visu)
                        {
                            pos1.WriteGPRMC(output);
                            //pos2.WriteGPRMC(output);
                        }
                        else
                        {
                            output.WriteLine(pos1.Latitude + "," + pos1.Longitude);
                            //output.WriteLine(pos2.Latitude + "," + pos2.Longitude);
                        }
                        // On prend la nouvelle valeur comme référence
                        AncBear = NewBear;
                    }
                    else
                    {
                        this.keep = false;
                    }
                }
                else
                {
                    //Seuil en degrees
                    if (NewBear <= (AncBear - this.seuil) || NewBear >= (AncBear + this.seuil))
                    {
                        this.keep = true;
                        //On écrit dans le fichier
                        if (visu)
                        {
                            pos1.WriteGPRMC(output);
                            //pos2.WriteGPRMC(output);
                        }
                        else
                        {
                            output.WriteLine(pos1.Latitude + "," + pos1.Longitude);
                            //output.WriteLine(pos2.Latitude + "," + pos2.Longitude);
                        }
                        // On prend la nouvelle valeur comme référence
                        AncBear = NewBear;
                    }
                    else
                    {
                        this.keep = false;
                    }
                }
            }
        } 

   /*     public void Optimize(List<GpsPosition> liste)
        {
            if (pos1.Latitude != pos2.Latitude && pos1.Longitude != pos2.Longitude)
            {
                this.NewBear = (Double)this.Bearing();

                if (NewBear <= AncBear * (1 - this.seuil) || NewBear >= AncBear * (1 + this.seuil))
                {
                    this.keep = true;
                    AncBear = NewBear;
                }
                else
                {
                    this.keep = false;
                }
            }
        }*/
     
    }

}
