﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;
using System.Globalization;

using Microsoft.WindowsMobile.Samples.Location;


namespace GpsSample
{
    public class Gpx
    {
        internal class Coordinate
        {
            private double _latitude = 0;
            private double _longitude = 0;
            private double _altitude = 0;
            private DateTime _coordinateTime;
            private bool notime = false;

            public bool Notime
            {
                get { return notime; }
            }

            public Coordinate(GpsPosition pos)
            {
                this._latitude = pos.Latitude;
                this._longitude = pos.Longitude;
                if (pos.SeaLevelAltitude>0.0)
                     this._altitude = pos.SeaLevelAltitudeValid ? pos.SeaLevelAltitude : 0;
                try
                {
                    this._coordinateTime = pos.Time;
                    this.notime = false;
                }
                catch 
                {
                    this.notime = true;
                    this._coordinateTime = new DateTime();
                }
            }

            public double Altitude
            {
                get { return _altitude; }
                set { _altitude = value; }
            }

            public double Latitude
            {
                get { return _latitude; }
                set { _latitude = value; }
            }

            public double Longitude
            {
                get { return _longitude; }
                set { _longitude = value; }
            }

            public DateTime CoordinateTime
            {
                get { return _coordinateTime; }
                set { _coordinateTime = value; }
            }
        }
        //private List<Coordinate> _waypoints = new List<Coordinate>();
        //private List<Coordinate> _routepoints = new List<Coordinate>();

        private Coordinate _c;

        private XmlTextWriter _gpxXml;

        private TextWriter _output;

        public TextWriter Output
        {
            get { return _output; }
            set { _output = value; }
        }

        private string _name = String.Empty;

        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        public Gpx(GpsPosition pos, TextWriter outp)
        {
            this._c = new Coordinate(pos);
            this._output = outp;
            GetGpx();
        }

        public Gpx(TextWriter outp)
        {
            this._output = outp;
            GetGpx();
        }

        public void SetPos(GpsPosition pos) 
        {
            this._c = new Coordinate(pos);
        }

        /*public void AddWaypoint(Coordinate waypoint)
        {
            _waypoints.Add(waypoint);
        }*/

      /*  public void AddRoutepoint(Coordinate routepoint)
        {
            _routepoints.Add(routepoint);
        }*/

        public void GetGpx()
        {
            if (this._gpxXml == null)
            {
                this._gpxXml = new XmlTextWriter(this._output);
                this._gpxXml.WriteStartDocument();
                this._gpxXml.WriteStartElement("gpx");
                this._gpxXml.WriteAttributeString("version", "1.0");
                this._gpxXml.WriteAttributeString("creator", "GPSSample");
                this._gpxXml.WriteAttributeString("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
                this._gpxXml.WriteAttributeString("xmlns", "http://www.topografix.com/GPX/1/0");
                this._gpxXml.WriteAttributeString("xsi:schemaLocation", "http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd");
                this._gpxXml.WriteStartElement("trk");
                this._gpxXml.WriteElementString("name", _name);
                this._gpxXml.WriteStartElement("trkseg");
            }
            else
            {
                this.WriteRoute();
            }
            

            // START: Waypoint
            /*if (_waypoints != null)
            {
                foreach (Coordinate c in _waypoints)
                {
                    gpxXml.WriteStartElement("wpt");
                    WriteWptType(gpxXml, c);
                    gpxXml.WriteEndElement();
                }
            }*/
            // END: Waypoint

            // START: Route
            //if ((_routepoints != null) && (_routepoints.Count > 0))
            //{
                
        }

        public void WriteRoute()
        {
            
            //this._gpxXml.WriteElementString("name", _name);
            // START: Route points
            //foreach (Coordinate c in _routepoints)
            //{
            this._gpxXml.WriteStartElement("trkpt");
            WriteWptType();
            this._gpxXml.WriteEndElement();
            //}
            // END: Route points
        }

        public void CloseFile()
        {
            //this._gpxXml.WriteEndElement();
            //}
            // END: Route

            this._gpxXml.WriteEndElement();
            this._gpxXml.Close();
            this._output.Close();
        }

        private void WriteWptType()
        {
            this._gpxXml.WriteAttributeString("lat", this._c.Latitude.ToString(new CultureInfo("en-US")));
            this._gpxXml.WriteAttributeString("lon", this._c.Longitude.ToString(new CultureInfo("en-US")));
            if (this._c.Altitude > 0)
                this._gpxXml.WriteElementString("ele", this._c.Altitude.ToString(new CultureInfo("en-US")));
            else
                this._gpxXml.WriteElementString("ele", String.Empty);
            if (!(this._c.Notime))
                this._gpxXml.WriteElementString("time", ConvertToUtcTimeString(this._c.CoordinateTime));
            else
                this._gpxXml.WriteElementString("time", String.Empty);
        }

        public static string ConvertToUtcTimeString(DateTime t)
        {
            TimeSpan offset = System.TimeZone.CurrentTimeZone.GetUtcOffset(t);
            string offsetString = "";
            if (offset.Hours == 0)
            {
                offsetString = "Z";
            }
            else if (offset.ToString().StartsWith("-"))
            {
                offsetString = offset.ToString().Substring(0, 6);
            }
            else
            {
                offsetString = "+" + offset.ToString().Substring(0, 5);
            }

            return t.ToUniversalTime().ToString("s") + offsetString;
        }
    }
}